{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}validation-error-modal{% endblock %}
{% block modal_title %}Validation Error{% endblock %}

{% block modal_body %}
<div id="errorMessage"></div>
{% endblock %}

{% block modal_footer %}
<button class="btn btn-secondary"{% if BOOTSTRAP_VERSION|first == "4" %}data-dismiss{% elif BOOTSTRAP_VERSION|first == "5"  %}data-bs-dismiss{% endif %}="modal">
    <i class="fas fa-times"></i> Close
</button>
{% endblock %}