<div>
    <button id="btn-previous-page" type="button" class="btn btn-secondary {% if BOOTSTRAP_VERSION|first == "4" %}float-right{% elif BOOTSTRAP_VERSION|first == "5"  %}float-end{% endif %}">
        <i class="fas fa-arrow-left"></i> Back to previous
    </button>
</div>